import { Container, EventEmitter, Graphics, Text } from 'pixi.js'
import { Scale } from './scale'
import type { GameState } from './interfaces'
import type { Tile } from './Tile'
import { getColorBackground } from './backgrounds'

export class Board extends EventEmitter {
  container: Container
  height: number
  width: number
  scaleY: Function
  scaleX: Function
  state: GameState | undefined
  initialContainer: Container = new Container()
  mainContainer: Container = new Container()
  leftContainer: Container = new Container()
  rightContainer: Container = new Container()
  tiles: Tile[] = []
  boneyard: Tile[] = []

  constructor(container: Container, canvas: HTMLCanvasElement) {
    super()
    this.container = container
    this.height = canvas.height - 130
    this.width = canvas.width
    this.container.width = this.width
    this.container.height = this.height
    this.container.y = 0
    const scaleXSteps = Math.floor(this.width / 25)
    const scaleYSteps = Math.floor(this.height / 25)
    this.scaleX = Scale([-scaleXSteps, scaleXSteps], [0, this.width])
    this.scaleY = Scale([-scaleYSteps, scaleYSteps], [0, this.height])

    this.initialContainer = new Container()
    this.initialContainer.addChild(
      new Text({
        text: 'Initial Board',
        style: {
          fontFamily: 'Arial',
          fontSize: 12,
          fill: 0xff1010,
          align: 'center'
        }
      })
    )
    this.container.addChild(this.initialContainer)
    this.mainContainer = new Container()
    this.mainContainer.addChild(
      new Text({
        text: 'Main Board',
        style: {
          fontFamily: 'Arial',
          fontSize: 12,
          fill: 0xff1010,
          align: 'center'
        }
      })
    )
    this.container.addChild(this.mainContainer)
    this.createLeftRightContainers()
    this.initialContainer.visible = false
    this.mainContainer.visible = false
  }

  private createLeftRightContainers() {
    console.log('this.height :>> ', this.height)
    this.leftContainer.x = 0
    this.leftContainer.y = 0
    this.leftContainer.width = 50
    this.leftContainer.height = this.height
    this.leftContainer.eventMode = 'static'
    this.leftContainer.cursor = 'pointer'

    const leftSide = new Graphics()
      .rect(0, 0, 50, this.height)
      // Draw the wheel
      .fill({ color: 0x848484 })
    leftSide.alpha = 0
    this.leftContainer.addChild(leftSide)
    this.leftContainer.on('pointerover', () => {
      leftSide.alpha = 0.5
    })
    this.leftContainer.on('pointerout', () => {
      leftSide.alpha = 0
    })
    this.leftContainer.on('pointerdown', () => {
      this.emit('leftClick')
    })

    this.leftContainer.addChild(
      new Text({
        text: 'Left',
        style: {
          fontFamily: 'Arial',
          fontSize: 12,
          fill: 0xff1010,
          align: 'center'
        }
      })
    )
    //
    this.rightContainer.x = this.width - 50
    this.rightContainer.y = 0
    this.rightContainer.width = 50
    this.rightContainer.height = this.height
    this.rightContainer.eventMode = 'static'
    this.rightContainer.cursor = 'pointer'

    const rightSide = new Graphics()
      .rect(0, 0, 50, this.height)
      // Draw the wheel
      .fill({ color: 0x848484 })
    rightSide.alpha = 0
    this.rightContainer.addChild(rightSide)
    this.rightContainer.on('pointerover', () => {
      rightSide.alpha = 0.5
    })
    this.rightContainer.on('pointerout', () => {
      rightSide.alpha = 0
    })
    this.rightContainer.on('pointerdown', () => {
      this.emit('rightClick')
    })
    this.rightContainer.addChild(
      new Text({
        text: 'Right',
        style: {
          fontFamily: 'Arial',
          fontSize: 12,
          fill: 0xff1010,
          align: 'center'
        }
      })
    )

    this.container.addChild(this.leftContainer)
    this.container.addChild(this.rightContainer)
  }

  setState(state: GameState) {
    this.state = state
  }
}
