import {
  Application,
  Container,
  EventEmitter,
  Graphics,
  Rectangle,
  RoundedRectangle,
  Sprite,
  Text,
  Texture
} from 'pixi.js'
import { Tile } from './Tile'
import type { PlayerState } from './interfaces'

export default class Hand extends EventEmitter {
  tiles: Tile[] = []
  container: Container
  height: number
  width: number

  constructor(container: Container, canvas: HTMLCanvasElement) {
    super()
    this.container = container
    this.height = 130
    this.width = canvas.width
    this.container.y = canvas.height - this.height
    this.container.width = this.width
    this.container.height = this.height
    this.addBg()
    this.createPassButton()
  }

  setTiles(playerState: PlayerState) {
    this.tiles = this.createTiles(playerState)
  }

  private addBg() {
    console.log('addBg')
    const bg = new Sprite(Texture.WHITE)
    bg.alpha = 0.05
    bg.width = this.width
    bg.height = this.height
    this.container.addChild(bg)
    this.container.addChild(
      new Text({
        text: 'Hand',
        style: {
          fontFamily: 'Arial',
          fontSize: 12,
          fill: 0xff1010,
          align: 'center'
        }
      })
    )
  }

  private onClick(tile: Tile) {
    const selected = this.tiles.find((t) => t.selectd)
    if (selected) {
      selected.y = selected.y + 10
      selected.selectd = false
    }

    tile.selectd = true
    tile.y = tile.y - 10
    this.emit('tileClick', { id: tile.id })
  }

  private createPassButton() {
    const rectangle = new Graphics().roundRect(0, 0, 80, 30, 10).fill(0xffff00)

    const text = new Text({
      text: 'PASS',
      style: {
        fontFamily: 'Arial',
        fontSize: 12,
        fontWeight: 'bold',
        fill: 0x121212,
        align: 'center'
      }
    })
    text.anchor = 0.5

    const buttonPassContainer = new Container()

    buttonPassContainer.addChild(rectangle)
    buttonPassContainer.addChild(text)

    text.y = buttonPassContainer.height / 2 - 4
    text.x = buttonPassContainer.width / 2 - 8

    buttonPassContainer.eventMode = 'static'
    buttonPassContainer.cursor = 'pointer'
    buttonPassContainer.x = 20
    buttonPassContainer.y = this.height / 2 - 10
    rectangle.alpha = 0.7
    text.alpha = 0.7
    buttonPassContainer.on('pointerdown', () => {
      this.emit('passClick')
    })
    buttonPassContainer.on('pointerover', () => {
      rectangle.alpha = 1
      text.alpha = 1
    })

    buttonPassContainer.on('pointerout', () => {
      rectangle.alpha = 0.7
      text.alpha = 0.7
    })

    this.container.addChild(buttonPassContainer)
  }

  private createTiles(playerState: PlayerState) {
    const deltaX = (playerState.hand.length * 50 - 5) / 2
    return playerState.hand.map((tile, i) => {
      const newTile: Tile = new Tile(tile.id, tile.pips)

      newTile.y = newTile.height / 2 + 20
      newTile.x = deltaX + newTile.width / 2 + i * (newTile.width + 5)
      newTile.eventMode = 'static'
      newTile.cursor = 'pointer'
      newTile.alpha = 0.7
      this.container.addChild(newTile)
      newTile.on('pointerdown', () => this.onClick(newTile))
      newTile.on('pointerover', () => (newTile.alpha = 1))
      newTile.on('pointerout', () => (newTile.alpha = 0.7))
      return newTile
    })
  }
}
